
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: EventRegistry, HideUIPanel, HIGHLIGHT_FONT_COLOR, MajorFactions_LoadUI, MajorFactionRenownFrame, RENOWN_LEVEL_LABEL, select, ShowUIPanel, string, ToggleCharacter

local BreakUpLargeNumbers = BreakUpLargeNumbers
local C_GossipInfo = C_GossipInfo
local C_MajorFactions = C_MajorFactions
local C_Reputation = C_Reputation
local floor = floor
local GetText = GetText
local GetWatchedFactionInfo = GetWatchedFactionInfo

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display Reputation")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI Reputation", {
	description = "Shows while a reputation bar's 'Show as Experience Bar' option is enabled.",
	type = "progress bar",
	label = UNKNOWN,
	level = UNKNOWN,
	text = UNKNOWN,
	value = "0",
	suffix = "%",
	visible = false,
})

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:RegisterEvent("MAJOR_FACTION_RENOWN_LEVEL_CHANGED", "Update")
	self:RegisterEvent("QUEST_ACCEPTED", "Update")
	self:RegisterEvent("QUEST_REMOVED", "Update")
	self:RegisterEvent("UPDATE_FACTION", "Update")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE", "OnConfigUpdate")
	self:Update()
end

function M:OnConfigUpdate(event, key, ...)
	local c, g, k = string.split(".", key)

	if (c == "color" and g == "reputation") or key == "color.resource.renown" then
		self:Update()
	end
end

function M:ShouldBeVisible()
	return GetWatchedFactionInfo() ~= nil
end

function M:Update()
	local name, reaction, min, max, value, factionID = GetWatchedFactionInfo()

	if not name then
		O.visible = false
		return
	end

	local color = C.REPUTATION_TO_COLOR[reaction]
	local level = GetText("FACTION_STANDING_LABEL" .. reaction, C.PLAYER_SEX)

	if C_Reputation.IsFactionParagon(factionID) then
		min, max, value, color = self:GetFactionParagonInfo(factionID, reaction)
	elseif C_Reputation.IsMajorFaction(factionID) then
		min, max, value, level, color = self:GetMajorFactionData(factionID)
	elseif C_GossipInfo.GetFriendshipReputation(factionID).friendshipFactionID > 0 then
		min, max, value, level = self:GetFriendshipReputation(factionID)
	end

	local percent = ((value - min) / (max - min)) * 100

	O.barR, O.barG, O.barB = color.r, color.g, color.b
	O.label = name
	O.level = color:WrapTextInColorCode(level or "")
	O.text = C.XP_TEXT:format(BreakUpLargeNumbers(value), BreakUpLargeNumbers(max), floor(percent))
	O.value = percent
	O.visible = M:ShouldBeVisible()
end

-- Get friendship reputation information for the given factionID and return the level, min, max, and value attributes.
function M:GetFriendshipReputation(factionID)
	local reputationInfo = C_GossipInfo.GetFriendshipReputation(factionID)
	local level = reputationInfo.reaction
	local min, max, value

	if reputationInfo.nextThreshold then
		min = reputationInfo.reactionThreshold
		max = reputationInfo.nextThreshold
		value = reputationInfo.standing
	else
		min = 0
		max = 1
		value = 1
	end

	return min, max, value, level
end

-- Get paragon reputation information for the given factionID and return the min, max, and value attributes. Update the value if there's a reward pending.
function M:GetFactionParagonInfo(factionID, reaction)
	local currentValue, threshold, _, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID)
	local min = 0
	local max = threshold
	local value = currentValue % threshold

	if hasRewardPending then
		value = value + threshold
	end

	local color = C_Reputation.IsMajorFaction(factionID) and C.RENOWN_RESOURCE_COLOR or C.REPUTATION_TO_COLOR[reaction]

	return min, max, value, color
end

-- Get major faction data for the given factionID and return the color, min, max, value, and level attributes.
function M:GetMajorFactionData(factionID)
	local majorFactionData = C_MajorFactions.GetMajorFactionData(factionID)

	local color = C.RENOWN_RESOURCE_COLOR
	local min = 0
	local max = majorFactionData.renownLevelThreshold
	local value = majorFactionData.renownReputationEarned
	local level = RENOWN_LEVEL_LABEL .. majorFactionData.renownLevel

	return min, max, value, level, color
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	if button == "RightButton" then
		local factionID = select(6, GetWatchedFactionInfo())

		if factionID and C_Reputation.IsMajorFaction(factionID) then
			MajorFactions_LoadUI()
			HideUIPanel(MajorFactionRenownFrame)
			EventRegistry:TriggerEvent("MajorFactionRenownMixin.MajorFactionRenownRequest", factionID)
			ShowUIPanel(MajorFactionRenownFrame)
			return
		end
	end

	ToggleCharacter("ReputationFrame")
end

function O.OnTooltipShow(tooltip)
	tooltip:SetText(O.label)
	tooltip:AddLine(O.text, HIGHLIGHT_FONT_COLOR:GetRGB())
	tooltip:AddLine(O.level)
end
